function [f,thetabest] = numr_f(A,thetamid,plotnum,fold,iter)
% Michael Overton and Emre Mengi (Last updated on March 9th 2005)
%
% called by numr.m
%
% Computes g(theta) = lambda_{max}(H(theta)) at each point theta in thetamid.
% Returns f = max{g(theta)| theta in thetamid} and thetabest satisfying 
% g(thetabest) = f (see the comments of the function nr_2way for the 
% definition of H(theta)).
%
% Input:
%       A           : Input matrix(must be square).
%       thetamid    : Set of midangles
%       plotnum     : Figure number.
%       fold        : Previous estimate of the numerical radius.
%       iter        : Number of updates performed until this call. 
%
% Output:
%       f           : Maximum function value among the function
%                     values at midangles.
%       thetabest   : One of the midangles at which the function
%                     value is maximum.


% get the size of the input matrix and check whether it's square or not.
n = length(A);
if size(A) ~= [n n]
   error('numr_f: A must be square')
end

for j = 1:length(thetamid)
    % compute g(theta) at the jth midpoint in thetamid
	H = 0.5*((cos(thetamid(j))+i*sin(thetamid(j)))*A + ...
            (cos(thetamid(j))-i*sin(thetamid(j)))*A');
	eH = eig(H);    

	fmax(j) = max(real(eH));
end

% max of max eigenvalues
[maxval,maxind] = max(fmax);

if iter == 1
    fold = maxval-.5;
end

if plotnum > 0 
    figure(plotnum);
    if thetamid(maxind) < 0
        thetat = thetamid(maxind) + 2*pi;
    else
        thetat = thetamid(maxind);
    end
    plot(thetat, fold, 'bx')
    plot([thetat thetat], [fold maxval], 'm-')
    plot(thetat, maxval, 'g+')
end

% set the return variables
f = maxval;
thetabest = thetamid(maxind);
return;