function E = mypolyeig(K)
% Copyright by Emre Mengi
% call : E = mypolyeig(K)
%
% task : computes the eigenvalues of the matrix
%        polynomial sum_j=0^m lambda^j K(:,:,(j+1)).
%        Note that the matlab function polyeig
%        requires each of the coefficient matrices
%        as an input argument. However, in our applications
%        we don't know the degree of the polynomial
%        a priori, so the input parameter pattern
%        for polyeig makes it unusable for our purpose.
%
% Input :
%   K     :  Sequence of matrices. Each of the matrices
%            K(:,:,j) in the sequence must be square.
%            The number of matrices in the sequence minus
%            one is the degree of the input polynomial.
%
% Output :
%   E     : Eigenvalues of the matrix polynomial.

[n,nc,mk] = size(K);

m = mk-1;

if n ~= nc
    error('Each matrix in the sequence must be square');
end

% linearize the matrix polynomial
A = zeros(n*m,n*m);
In = eye(n);
for j = 1:m-1
    A(1+(j-1)*n:j*n,1+j*n:(j+1)*n) = In;
    A(1+(m-1)*n:m*n,1+(j-1)*n:j*n) = -K(:,:,j);
end
A(1+(m-1)*n:m*n,1+(m-1)*n:m*n) = -K(:,:,m);


B = zeros(n*m,n*m);
for j = 1:m-1
    B(1+(j-1)*n:j*n,1+(j-1)*n:j*n) = In;
end
B(1+(m-1)*n:m*n,1+(m-1)*n:m*n) = K(:,:,m+1);


% solve the generalized eigenvalue problem
E = eig(A,B);
return;