function [] = plot_num_rad(A,plotfig,lowr,highr,type)
% Emre Mengi and Michael Overton (Last updated on March 9th 2005)
% call : plot_num_rad(A,plotfig,lowr,highr,type)
%   task
%                 Plots either the function f(theta)=lambda_{max}(H(theta))               
%                 or the function f'(theta)=lambda_{maxmod}(H(theta))
%                 where H(theta)=(Ae^{itheta) + A'e^{-itheta))/2 and
%                 theta in [lowr,highr). 
%
%   input
%       A       : Input matrix (must be square).
%       plotfig : figure handle, if no figure with this handle
%                 exists, a new figure will be opened, otherwise
%                 the circle will be drawn on the figure with
%                 handle plotfig.
%       lowr    : Lower bound for the domain of the function f or f'.
%       highr   : Upper bound for the domain of the function f or f' (highr >= lowr).
%       type    : 0: plots f (default).
%                 1: plots f'.
%       
%
%   output
%                 no output


% default
if (nargin < 5)
    type = 0;
end


% error checks
if (highr < lowr)
    error('highr should not be less than lowr')
end


n = length(A);
if size(A) ~= [n n]
   error('A must be square')
end
In = eye(n);



% theta-axis is discretized, the length between two successive
% points is arranged so that there are 1000 points selected.
number_of_points = 1000;
% disc_int is the distance between two successive selected points on
% theta-axis.
disc_int = (highr - lowr) / number_of_points;


theta = lowr;
thetav = [];
fv = [];
while theta < highr
    H = (A*exp(i*theta) + A'*exp(-i*theta))/2;
    if type == 0
        f = max(real(eig(H)));
    else
        f = max(abs(real(eig(H))));
    end
    thetav = [thetav theta];
    fv = [fv f];
    theta = theta + disc_int;
end

figure(plotfig);
plot(thetav,fv,'r-');