function [xs,ys] = poly_ps(P,gamma,epsln1,epsln2, ...
                             epsln_length,int_length,fignum)
% Copyright Emre Mengi, July 2006
% call: draw_level_sets(P,gamma,epsln1,epsln2,epsln_length,int_length,fignum)
% Draws the epsln-pseudospectrum of the matrix polynomial P for
% epsln = epsln1, epsln1 + epsln_length, \dots, 
%         epsln1 + floor((epsln2-epsln1)/epsln_length)*epsln_length.
% The plot is generated by applying radial searches for various theta
% ona 1D grid. The positive real number int_length is the difference
% between two succesive angles on the grid. fignum is the handle for
% the figure.

[n,nc,k] = size(P);

k = k-1;

if n ~= nc
    error('the input polynomials must be square');
end


realtol = 10^-8;
smalltol = 10^-12;
In = eye(n);
Zn = zeros(n);

col = ['y.'; 'g.'; 'r.'; 'c.'; 'm.'; 'k.'; 'b.';...
       'y*'; 'g*'; 'r*'; 'c*'; 'm*'; 'k*'; 'b*'];
l = 1;

for epsln = epsln1:epsln_length:epsln2
    epsln
    fprintf('plotting the epsilon-pseudospectrum for delta=%f\n',epsln);
   

    xs = [];
    ys = [];
    for theta = 0:int_length:pi
    
     % for each theta perform a radial search
     
     % Create the matrix polynomial that needs to be solved
     for j = 0:2*k
        if j == 0
            Q(:,:,j+1) = [-epsln*gamma(j+1)^2*In P(:,:,j+1)'; ...
                            P(:,:,j+1)  -epsln*In];
        else
            if (j <= k)
                if mod(j,2) % j is odd, Qj is skew-Hermitian
                    Q(:,:,j+1) = [ Zn (-1)^((j+1)/2)*i*P(:,:,j+1)'*exp(-i*j*theta); ...
                            (-1)^((j+1)/2)*i*P(:,:,j+1)*exp(i*j*theta) Zn];
                else % j is even, Qj is Hermitian
                    Q(:,:,j+1) = [ epsln*gamma(j/2+1)^2*In*(-1)^(j/2+1) ...
                            (-1)^(j/2)*P(:,:,j+1)'*exp(-i*j*theta); ...
                            (-1)^(j/2)*P(:,:,j+1)*exp(i*j*theta) ...
                            Zn];
                end
            else
                if mod(j,2) % j is odd and greater than half of the degree
                    Q(:,:,j+1) = zeros(2*n,2*n);
                else % if j is even and greater than half of the degree
                    Q(:,:,j+1) = [ epsln*gamma(j/2+1)^2*In*(-1)^(j/2+1) Zn; ...
                                    Zn Zn];
                end
            end
        end
     end
        
     % compute the eigenvalues
     eigsQ = mypolyeig(Q);
     % extract the real eigenvalues
     imagind = find(abs(real(eigsQ)) < realtol & eigsQ ~= inf & eigsQ ~= -inf);
     imageigsQ = imag(eigsQ(imagind));
    
    
        %  keyboard
        % finally test whether the real eigenvalues are the moduli
        % of the intersection points with the level sets
        for j=1:length(imageigsQ)
            mins = compute_sing(P,gamma,imageigsQ(j)*exp(i*theta));
            
            % test succeeds: add the point to the set of points
            %                that are going to be plotted.
            if abs(mins-epsln) < smalltol
                xs = [xs real(imageigsQ(j)*exp(i*theta))];
                ys = [ys imag(imageigsQ(j)*exp(i*theta))];
            end
        end


    end

%    if ~isempty(xs)
     figure(fignum);
     hold on
     h = plot(xs,ys,col(l,:));
     if ~isempty(xs)
        strd(l,:) = sprintf('epsilon=%.4f',epsln);
        l = l+1;
    end
%    end
end

if l > 1
    legend(strd)
end

return;